/*
 *	configwindow.cpp	Implementation of the Config Window
 *
 *
 *
 */

// system includes
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>

// be includes
#include <Application.h>

// config app includes
#include "configwindow.h"

// driver includes
#include "sound.h"

// liblayout includes
#include "LayeredGroup.h"
#include "TabGroup.h"
#include "MBorder.h"
#include "HGroup.h"
#include "VGroup.h"

#include "MStringView.h"

#include "Space.h"
#include "MApplication.h"

// messages inside the Config Window
#define MSG_SWITCH_4_CHANNEL_DUPLICATE_MODE		'4chd'
#define MSG_SWITCH_SPDIF_IN_OUT_LOOPBACK		'siol'
#define MSG_SWITCH_SPDIF_OUT					'spdo'
#define MSG_SWITCH_SPDIF_IN_MONITOR				'sdim'
#define MSG_SWITCH_SPDIF_IN_OUT_COPYRIGHT		'sioc'
#define MSG_SWITCH_SPDIF_IN_VALIDITY			'spiv'
#define MSG_SWITCH_SPDIF_IN_FORMAT				'spif'
#define MSG_SWITCH_SPDIF_OUT_LEVEL				'spol'


// Constructor
ConfigWindow::ConfigWindow( char *WindowTitle, BRect Frame, int *fd )
	:MWindow( Frame, WindowTitle, B_TITLED_WINDOW, B_ASYNCHRONOUS_CONTROLS )
{
	this->fd = *fd;
	
	email = new URLView( "Marko Koscak", "mailto:beos@tisno.de", B_ALIGN_LEFT );
	email->AddAttribute( "META:name", "Marko Koscak" );
	email->AddAttribute( "META:nickname", "jobe" );
	email->AddAttribute( "META:url", "http://www.tisno.de" );
	email->AddAttribute( "META:city", "Aachen" );
	email->AddAttribute( "META:country", "Germany" );

	web = new URLView( "jobe's BeOS Page", "http://beos.tisno.de", B_ALIGN_LEFT );
	web->AddAttribute( "META:keyw", "jobe's Homepage" );

	ConfigView = new MBorder
	(
		M_RAISED_BORDER, 5, "config",
		new TabGroup (
			"Settings",
			new VGroup (
				new MBorder (
					M_LABELED_BORDER, 11, "SPDIF",
					new VGroup (
						SpdifInOutLoopback =
							new MCheckBox( "SPDIF-IN loopback to SPDIF-OUT (bypass)",
							new BMessage(MSG_SWITCH_SPDIF_IN_OUT_LOOPBACK) ),
						SwitchSpdifOut =
							new MCheckBox( "Enable SPDIF-OUT",
							new BMessage(MSG_SWITCH_SPDIF_OUT) ),
						MonitorSpdifIn =
							new MCheckBox( "Monitor SPDIF-IN (Digital SPDIF-IN signal pass to analog LINE-OUT)",
							new BMessage(MSG_SWITCH_SPDIF_IN_MONITOR) ),
						new MBorder (
							M_ETCHED_BORDER, 5, "",
							new HGroup (
								new MStringView( "SPDIF-OUT level:", B_ALIGN_CENTER ),
								SpdifOutLevel = new MRadioGroup (
									"5 Vp-p (Optical)",
									"0.5 Vp-p (Coaxial)",
									0
								),0
							)
						),
						new MBorder (
							M_ETCHED_BORDER, 5, "",
							new HGroup (
								new MStringView( "SPDIF-IN format:", B_ALIGN_CENTER ),
								SpdifInFormat = new MRadioGroup (
									new BMessage(MSG_SWITCH_SPDIF_IN_FORMAT),
									"Normal",
									"Inverse",
									0
								),0
							)
						),
						SpdifInOutCopyright =
							new MCheckBox( "Enable SPDIF IN/OUT Copyright protection",
							new BMessage(MSG_SWITCH_SPDIF_IN_OUT_COPYRIGHT) ),
						SpdifInValidity =
							new MCheckBox( "Enable SPDIF-IN validity detection",
							new BMessage(MSG_SWITCH_SPDIF_IN_VALIDITY) ),
						0
					)
				),
				new MBorder (
					M_LABELED_BORDER, 11, "Analog",
					new VGroup (
						Switch4ChannelDuplicateMode =
							new MCheckBox( "Enable 4 channels analog duplicate mode",
							new BMessage(MSG_SWITCH_4_CHANNEL_DUPLICATE_MODE) ),
						0
					)
				),0
			), // end VGroup from Settings
			"Info",
			new VGroup (
				new Space(),
				new MBorder (
					M_LABELED_BORDER, 11, "Onboard C-Media chip",
					new VGroup (
						new HGroup (
							new MStringView( "device ID:", B_ALIGN_RIGHT ),
							DeviceID = new MStringView( "n/a", B_ALIGN_LEFT ),0
						),
						new HGroup (
							new MStringView( "internal chip ID:", B_ALIGN_RIGHT ),
							ChipID = new MStringView( "n/a", B_ALIGN_LEFT ),0
						),
						new HGroup (
							new MStringView( "device driver Version:", B_ALIGN_RIGHT ),
							DriverVer = new MStringView( "n/a", B_ALIGN_LEFT ),0
						),0
					)
				),
				new Space(),
				new HGroup(
					new Space(),
					new MBorder (
						M_LABELED_BORDER, 11, "Contact the developer",
						new VGroup (
							new HGroup (
								new VGroup (
									new MStringView( "Web:", B_ALIGN_LEFT ),
									(MStringView *)web,0
								),
								new VGroup (
									new MStringView( "e-Mail:", B_ALIGN_LEFT ),
									(MStringView *)email,0
								),0
							),
							new Space( minimax(10, 10, 10, 10) ),
							new MStringView( "This program is using:", B_ALIGN_LEFT ),
							new MStringView( "liblayout from Marco Nelissen", B_ALIGN_LEFT ),
							new MStringView( "URLView from William Kakes", B_ALIGN_LEFT ),
							0
						)
					),
					new Space(),0
				),0 // end of HGroup
			),0 // end VGroup from Info
		) // end of TabGroup
	);
	
	// disable unfinished components
	SwitchSpdifOut->SetEnabled( false );
	
	AddChild( dynamic_cast<BView*>(ConfigView) );

	// get the values from the driver for the checkboxes, Radiobuttons, ...
	InitGUI();
}

// Destructor
ConfigWindow::~ConfigWindow()
{}

bool ConfigWindow::QuitRequested(void)
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
}

void ConfigWindow::InitGUI(void)
{
	// Read the conditions form the driver and set the GUI elements similarly
	
	// Tab Settings:
	// 0 - off
	// 1 - on
	int8 reg_value;
	
	// get SPDIF-IN loopback to SPDIF (bypass)
	reg_value = 0; // off is default
	if( ioctl( this->fd, SOUND_GET_SPDIF_IN_OUT_LOOPBACK, &reg_value, sizeof(reg_value) ) < 0 )
		printf( "ioctl (SOUND_GET_SPDIF_IN_OUT_LOOPBACK) failed.\n" );
	SpdifInOutLoopback->SetValue( reg_value );

	// get SPDIF-OUT enable
	// XXX
		
	// get SPDIF-IN monitor
	reg_value = 0; // off is default
	if( ioctl( this->fd, SOUND_GET_SPDIF_MONITOR, &reg_value, sizeof(reg_value) ) < 0 )
		printf( "ioctl (SOUND_GET_SPDIF_MONITOR) failed.\n" );
	MonitorSpdifIn->SetValue( reg_value );
	
	// get SPDIF-OUT level
	reg_value = 0; // off is default
	if( ioctl( this->fd, SOUND_GET_SPDIF_OUT_LEVEL, &reg_value, sizeof(reg_value) ) < 0 )
		printf( "ioctl (SOUND_GET_SPDIF_OUT_LEVEL) failed.\n" );
	SpdifOutLevel->SetActive( reg_value );
	
	// get SPDIF-IN format
	reg_value = 0; // off is default
	if( ioctl( this->fd, SOUND_GET_SPDIF_IN_FORMAT, &reg_value, sizeof(reg_value) ) < 0 )
		printf( "ioctl (SOUND_GET_SPDIF_IN_FORMAT) failed.\n" );
	SpdifInFormat->SetActive( reg_value );
	
	// get SPDIF-IN-OUT copyright protection
	reg_value = 0; // off is default
	if( ioctl( this->fd, SOUND_GET_SPDIF_IN_OUT_COPYRIGHT, &reg_value, sizeof(reg_value) ) < 0 )
		printf( "ioctl (SOUND_GET_SPDIF_IN_OUT_COPYRIGHT) failed.\n" );
	SpdifInOutCopyright->SetValue( reg_value );
	
	// get SPDIF-IN validity detection
	reg_value = 0; // off is default
	if( ioctl( this->fd, SOUND_GET_SPDIF_IN_VALIDITY, &reg_value, sizeof(reg_value) ) < 0 )
		printf( "ioctl (SOUND_GET_SPDIF_IN_VALIDITY) failed.\n" );
	SpdifInValidity->SetValue( reg_value );

	// get 4 channel analog duplicate mode 
	reg_value = 0; // off is default
	if( ioctl( this->fd, SOUND_GET_4_CHANNEL_DUPLICATE, &reg_value, sizeof(reg_value) ) < 0 )
		printf( "ioctl (SOUND_GET_4_CHANNEL_DUPLICATE) failed.\n" );
	Switch4ChannelDuplicateMode->SetValue( reg_value );
	
	// Tab Info:
	// get device ID
	int32 device_value = 0;
	if( ioctl( this->fd, SOUND_GET_DEVICE_ID, &device_value, sizeof(device_value) ) < 0 )
		printf( "ioctl (SOUND_GET_DEVICE_ID) failed.\n" );
	switch( device_value )
	{
	case 0x0100: // cmi8338 rev. A (0x0100)
		DeviceID->SetText( "cmi8338 rev. A (0x0100)");
		break;
	case 0x0101: // cmi8738 rev. B (0x0101)
		DeviceID->SetText( "cmi8338 rev. B (0x0101)");
		break;
	case 0x0111: // cmi8738 rev. A (0x0111)
		DeviceID->SetText( "cmi8738 rev. A (0x0111)");
		break;
	case 0x0112: // cmi8738 rev. B (0x0112)
		DeviceID->SetText( "cmi8738 rev. B (0x0112)");
		break;
	default:
		DeviceID->SetText( "unknown");		
		printf( "Device ID: %x, ??? please mail me! (beos@tisno.de)", (int16)device_value );
		break;
	}
	
	// get internal chip ID
	// XXX
	
	// get device driver version
	char DriverVersion[] = "x.x.x (mmm dd, yyyy)----------";
	if( ioctl( this->fd, SOUND_GET_DRIVER_VERSION, &DriverVersion, sizeof(DriverVersion) ) < 0 )
		printf( "ioctl (SOUND_GET_DRIVER_VERSION) failed.\n" );
	
//	BString(
	DriverVer->SetText( DriverVersion );

}

void ConfigWindow::MessageReceived( BMessage* msg )
{
	switch (msg->what)
	{
		case MSG_SWITCH_SPDIF_IN_OUT_LOOPBACK:
		{
			int8 reg_value = 0; //off
			if( SpdifInOutLoopback->Value() )
				 reg_value = 1; // on
			// set SPDIF-IN loopback to SPDIF-OUT (bypass)
			if( ioctl( this->fd, SOUND_SET_SPDIF_IN_OUT_LOOPBACK, &reg_value, sizeof(reg_value) ) < 0 )
				printf( "ioctl (SOUND_SET_SPDIF_IN_OUT_LOOPBACK) failed.\n" );			
			break;
		}
		case MSG_SWITCH_SPDIF_OUT:
			// XXX
			break;
		case MSG_SWITCH_SPDIF_IN_MONITOR:
		{
			int8 reg_value = 0; //off
			if( MonitorSpdifIn->Value() )
				 reg_value = 1; // on
			// set SPDIF-IN Monitor (CDPlay)
			if( ioctl( this->fd, SOUND_SET_SPDIF_MONITOR, &reg_value, sizeof(reg_value) ) < 0 ) {
				printf( "ioctl (SOUND_SET_SPDIF_MONITOR) failed.\n" );
			}	
			break;
		}
		case MSG_SWITCH_SPDIF_OUT_LEVEL:
		{
			int8 reg_value = (int8)SpdifOutLevel->ActiveButton();

			printf( "Level status: %i (0 = Normal, 1 = Inverse)\n", reg_value );
			// 0 = Optical
			// 1 = Coaxial
			// set SPDIF-OUT level
			if( ioctl( this->fd, SOUND_SET_SPDIF_OUT_LEVEL, &reg_value, sizeof(reg_value) ) < 0 ) {
				printf( "ioctl (SOUND_SET_SPDIF_OUT_LEVEL) failed.\n" );
			}
			break;
		}
		case MSG_SWITCH_SPDIF_IN_FORMAT:
		{
			int8 reg_value = (int8)SpdifInFormat->ActiveButton();

			printf( "Format status: %i (0 = Normal, 1 = Inverse)\n", reg_value );
			// 0 = Normal
			// 1 = Inverse
			// set SPDIF-IN format
			if( ioctl( this->fd, SOUND_SET_SPDIF_IN_FORMAT, &reg_value, sizeof(reg_value) ) < 0 ) {
				printf( "ioctl (SOUND_SET_SPDIF_IN_FORMAT) failed.\n" );
			}	
			break;
		}
		case MSG_SWITCH_SPDIF_IN_OUT_COPYRIGHT:
		{		
			int8 reg_value = 0; //off
			if( SpdifInOutCopyright->Value() )
				 reg_value = 1; // on
			// set SPDIF-IN-OUT copyright protection
			if( ioctl( this->fd, SOUND_SET_SPDIF_IN_OUT_COPYRIGHT, &reg_value, sizeof(reg_value) ) < 0 ) {
				printf( "ioctl (SOUND_SET_SPDIF_IN_OUT_COPYRIGHT) failed.\n" );
			}
			break;
		}
		case MSG_SWITCH_SPDIF_IN_VALIDITY:
		{
			int8 reg_value = 0; //off
			if( SpdifInValidity->Value() )
				 reg_value = 1; // on
			// set SPDIF-IN validity detection
			if( ioctl( this->fd, SOUND_SET_SPDIF_IN_VALIDITY, &reg_value, sizeof(reg_value) ) < 0 ) {
				printf( "ioctl (SOUND_SET_SPDIF_IN_VALIDITY) failed.\n" );
			}
			break;
		}
		case MSG_SWITCH_4_CHANNEL_DUPLICATE_MODE:
		{
			int8 reg_value = 0; //off
			if( Switch4ChannelDuplicateMode->Value() )
				 reg_value = 1; // on
			// set 4 channel analog duplicate mode 
			if( ioctl( this->fd, SOUND_SET_4_CHANNEL_DUPLICATE, &reg_value, sizeof(reg_value) ) < 0 ) {
				printf( "ioctl (SOUND_SET_4_CHANNEL_DUPLICATE) failed.\n" );
			}
			break;
		}

/*	case ... :
		
		break;
*/
		default :
			MWindow::MessageReceived( msg );
			break;
		
	}
}
